#include <stdlib.h>
#include <string.h>
#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h> 

#include "main.h"
#include "lcd.h"
#include "lcd_draw.h"
#include "key.h"
#include "menu.h"
#include "valout.h"

volatile uint8_t cur_mode = 0;

uint16_t val_cur[3] = { 0xffff, 0xffff, 0xffff };
uint16_t val_set[3] = { 0x0000, 0x0000, 0x0000 };
const char val_unit[] = { 'A', 'V', 'W' };
const uint16_t powten[5] = {
    10000,
     1000,
      100,
       10,
        1
};

int main (void) {
    // taster eingang + pullup
    DDRD &= ~((1 << DDD2) | (1 << DDD4) | (1 << DDD7));
    PORTD |= (1 << PD2) | (1 << PD4) | (1 << PD7);

    // led
    LED_DDR |= (1<<LED_PIN);
    LED_PORT |= (1<<LED_PIN);

    // lcd start
    lcd_init(LCD_DISP_ON);
    lcd_clrscr();

    // enable interrupt/timer
    TCCR0A = (1<<WGM01);                // CTC
    TCCR0B = (1<<CS02) | ( 1<<CS00);    // prescale 1024
    OCR0A = (F_CPU / 1024 * 10e-3 + 0.5);    // every 10ms
    TIMSK0 = 1<<OCIE0A;                       // enable timer overflow a interrupt

    sei();

    // greeting
    lcd_puts_P("HELLO\n");
    for(uint8_t s = 0; s<=7; s++) {
        lcd_putc(s);
    }
    _delay_ms(1000);
    LED_PORT &= ~(1<<LED_PIN);

    lcd_clrscr();


    uint8_t state = 0;
    static uint8_t menuepos = 2;
    while(1) {
        // menue system
        switch( state ) {
            case STATE_NORM_INIT:
                state = state_norm_init();
            break;
            case STATE_NORM_WAITKEY:
                state = state_norm_waitkey();
            break;

            case STATE_MENU_INIT:
                state = state_menu_init( &menuepos );
            break;
            case STATE_MENU_CHANGED:
                state = state_menu_changed( &menuepos );
            break;
            case STATE_MENU_WAITKEY:
                state = state_menu_waitkey( &menuepos );
            break;

            case STATE_VAL_INIT:
                state = state_val_init( &menuepos );
            break;
            case STATE_VAL_CHANGED:
                state = state_val_changed();
            break;
            case STATE_VAL_WAITKEY:
                state = state_val_waitkey( &menuepos );
            break;

            default:
                state = STATE_NORM_INIT;
            break;  
        }
        lastline();
    }
}

void lastline() {
    static uint8_t llstate = 0;
    if ((toggletimer >> TT1s && 0x01 ) == ( llstate & 0x01 )) {
        if ( ( llstate & 0x06 ) == 0 ) { 
            llstate |= 0x04;   // reset to xx xx x1 0x
        } else {
            // xx xx x0 1x
            // xx xx x0 0x
            // dann reset
            llstate -= 2;
        }
        llstate ^= 0x01; 
        lcd_gotoxy(1, 1);
        lcd_uint16_unit( val_cur[  llstate >> 1 ],
                         val_unit[ llstate >> 1 ] );
    } 
    sei();
}

uint8_t state_norm_init( void ){
    lcd_gotoxy(0, 0);
    lcd_putc(0xa5);
    lcd_uint16_unit( val_set[ (cur_mode & CUR_MODE_CONST_MASK) >> CUR_MODE_CONST_SHIFT ],
                     val_unit[ (cur_mode & CUR_MODE_CONST_MASK) >> CUR_MODE_CONST_SHIFT ] );
    return STATE_NORM_WAITKEY;
}

uint8_t state_norm_waitkey( void ){
    if( get_key_press ( 1<<KEY0 )) {
        cur_mode ^= CUR_MODE_IS_ACTIVE;
        LED_PORT ^= (1<<LED_PIN);
    }
    if( get_key_long( 1<<KEY1 )){  
        return STATE_VAL_INIT;
    }
    if( get_key_press ( 1<<KEY2 )) {
        return STATE_MENU_INIT;
    }
    return STATE_NORM_WAITKEY;            
}

uint8_t state_menu_init( uint8_t *mp ){
    *mp = 0;
    return STATE_MENU_CHANGED;
}

uint8_t state_menu_changed( const uint8_t *mp ){
    lcd_gotoxy(0, 0);
    draw_menue( *mp );
    return STATE_MENU_WAITKEY;
}

uint8_t state_menu_waitkey( uint8_t *mp ){
    if( get_key_press ( 1<<KEY0 )) {
        *mp = menue_prev( *mp );
        return STATE_MENU_CHANGED;
    }
    if( get_key_press ( 1<<KEY2 )) {
        *mp = menue_next( *mp );
        return STATE_MENU_CHANGED;
    }
    if( get_key_short( 1<<KEY1 )){  
        // TODO: menu item action
    }
    if( get_key_long( 1<<KEY1 )) return STATE_NORM_INIT;
    return STATE_MENU_WAITKEY;
}

uint8_t state_val_init( uint8_t *mp ){
    lcd_command(LCD_DISP_ON_CURSOR_BLINK);
    *mp = 1;
    return STATE_VAL_CHANGED;
}

uint8_t state_val_changed( void ){
    lcd_gotoxy(0,0);
    lcd_putc('#');
    lcd_uint16_unit( val_set[ (cur_mode & CUR_MODE_CONST_MASK) >> CUR_MODE_CONST_SHIFT ],
                     val_unit[ (cur_mode & CUR_MODE_CONST_MASK) >> CUR_MODE_CONST_SHIFT ] );
    return STATE_VAL_WAITKEY;
}

uint8_t state_val_waitkey( uint8_t *mp ){
    // 1-5 => 1,2,4,5,6
    // use cursor to hilight current position
    if ( *mp >= 3 ) {
        // 3 is decimal dot
        lcd_gotoxy(*mp +1, 0);
    } else {
        lcd_gotoxy(*mp, 0);
    }
    if( get_key_short( 1<<KEY1 )) {
        *mp = *mp + 1;
        if ( *mp == 6 ) {
            lcd_command(LCD_DISP_ON);
            return STATE_NORM_INIT;
        }
    }
    if( get_key_long( 1<<KEY1 )) {
        lcd_command(LCD_DISP_ON);
        return STATE_NORM_INIT;
    }
    if( get_key_press ( 1<<KEY0 )) {
        val_set[ (cur_mode & CUR_MODE_CONST_MASK) >> CUR_MODE_CONST_SHIFT ] += powten[*mp-1];
        return STATE_VAL_CHANGED;
    }
    if( get_key_press ( 1<<KEY2 )) {
        val_set[ (cur_mode & CUR_MODE_CONST_MASK) >> CUR_MODE_CONST_SHIFT ] -= powten[*mp-1];
        return STATE_VAL_CHANGED;
    }
    return STATE_VAL_WAITKEY;
}
